@@ -1,4 +1,7 @@
-#' @title Returns parameters for all jobs as the rows of a data.frame.
+#' @title Retrieve Job Parameters.
+#'
+#' @description
+#' Returns parameters for all jobs as the rows of a data.frame.
 #'
 #' @template arg_reg
 #' @template arg_ids
@@ -7,13 +10,13 @@
 #' @examples
 #' # see batchExpandGrid
 getJobParamDf = function(reg, ids) {
+  checkRegistry(reg, strict = TRUE, writeable = FALSE)
   syncRegistry(reg)
   if (!missing(ids))
     ids = checkIds(reg, ids)
   tab = dbGetExpandedJobsTable(reg, ids, cols = c("job_id", "pars"))
-  # rownames are set by db* call
-  # unserialize parameters
-  res = convertListOfRowsToDataFrame(lapply(tab$pars,
-      function(x) unserialize(charToRaw(x))), row.names = rownames(tab))
-  return(dropNamed(res, "job_id"))
+  res = deserialize(tab$pars)
+  setDF(res, rownames = rownames(tab))
+
+  return(res)
 }
